/* disks-systray.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Gtk;
using ConfigH;
using UEvt;

namespace UDisks
{

    public class DisksTray : Gtk.StatusIcon
    {
        public DisksTray()
        {
            this.set_from_gicon(new ThemedIcon("drive-removable-media"));
            this.set_tooltip_text("UEvt DisksTray");
            this.visible = true;
            this.popup_menu.connect(popup_menu_callback);
        }

        private async void mount_from_menu(ObjectPath dev_path)
        {
            UDisks.Device dev;
            string path;

            try {
                dev = Bus.get_proxy_sync(BusType.SYSTEM,
                                "org.freedesktop.UDisks",
                                            dev_path);
            } catch (IOError io) {
                return;
            }

            /* if device is already mounted just notify the user */
            if(dev.device_is_mounted) {
                run_notification("Mount Device",
                    "Device %s already mounted.".printf(dev.device_file),
                    ICONDATADIR + "devices/drive-removable-media-2.png",
                    config.UEVT_NOTIFY_DISKS);
                return;
            }

            try {
                path = yield dev.filesystem_mount("", {"auth_no_user_interaction"});
            } catch (IOError io1) {
                warning("Cannot mount %s : %s", dev_path, io1.message);
                if(io1.message.contains("org.freedesktop.UDisks.Error.PermissionDenied"))
                        run_notification("Cannot Mount Device",
                            "Permission denied, check your PolicyKit authorizations.",
                            ICONDATADIR + "actions/dialog-disable.png",
                            config.UEVT_NOTIFY_DISKS);
                return;
            }

            message("Device %s mounted to %s", dev_path, path);
            run_notification("Mount Device",
                "Device %s has been mounted to %s.".printf(dev.device_file, path),
                ICONDATADIR + "devices/drive-removable-media-2.png",
                config.UEVT_NOTIFY_DISKS);
        }

        private async void umount_from_menu(ObjectPath dev_path)
        {
           UDisks.Device dev;

           try {
                dev = Bus.get_proxy_sync(BusType.SYSTEM,
                                "org.freedesktop.UDisks",
                                            dev_path);
            } catch (IOError io) {
                return;
            }

            /* if device is not mounted just notify the user */
            if(!dev.device_is_mounted) {
                run_notification("Unmount Device",
                    "Device %s already unmounted.".printf(dev.device_file),
                    ICONDATADIR + "devices/drive-removable-media-2.png",
                    config.UEVT_NOTIFY_DISKS);
                return;
            }

            try {
                yield dev.filesystem_unmount({});
            } catch (IOError io1) {
                warning("Cannot unmount %s : %s", dev_path, io1.message);
                if(io1.message.contains("org.freedesktop.UDisks.Error.Busy"))
                        run_notification("Cannot Unmount Device",
                            "Device is busy.",
                            ICONDATADIR + "actions/dialog-disable.png",
                            config.UEVT_NOTIFY_DISKS);
                return;
            }

            message("Device %s unmounted", dev_path);
            run_notification("Unmount Device",
                "Device %s has been unmounted.".printf(dev.device_file),
                ICONDATADIR + "devices/drive-removable-media-2.png",
                config.UEVT_NOTIFY_DISKS);
        }

        private async void open_from_menu(ObjectPath dev_path)
        {
           UDisks.Device dev;

           try {
                dev = Bus.get_proxy_sync(BusType.SYSTEM,
                                "org.freedesktop.UDisks",
                                            dev_path);
            } catch (IOError io) {
                return;
            }

            if(!dev.device_is_mounted)
                return;

            run_command_config(config.UEVT_GROUP_DISKS,
                    config.UEVT_OPEN_DEVICE,
                    dev.device_file, dev.device_mount_paths[0],
                    false);
        }

        private async void play_from_menu(ObjectPath dev_path)
        {
           UDisks.Device dev;

           try {
                dev = Bus.get_proxy_sync(BusType.SYSTEM,
                                "org.freedesktop.UDisks",
                                            dev_path);
            } catch (IOError io) {
                return;
            }

            run_command_config(config.UEVT_GROUP_DISKS,
                    config.UEVT_OPEN_AUDIO_DISC,
                    dev.device_file, "", false);
        }

        private async void blank_from_menu(ObjectPath dev_path)
        {
           UDisks.Device dev;

           try {
                dev = Bus.get_proxy_sync(BusType.SYSTEM,
                                "org.freedesktop.UDisks",
                                            dev_path);
            } catch (IOError io) {
                return;
            }

            run_command_config(config.UEVT_GROUP_DISKS,
                    config.UEVT_OPEN_BLANK_DISC,
                    dev.device_file, "", false);
        }

        private void about_clicked()
        {
            string authors[2];
            authors[0] = "Boris Albar";
            authors[1] = null;

            var about = new AboutDialog();
            about.set_version(PACKAGE_VERSION);
            about.set_program_name("UEvt DisksTray");
            about.set_license(licence);
            about.set_authors(authors);
            Image img = new Image.from_file(ICONDATADIR + "devices/drive-removable-media-2.png");
            about.set_logo(img.get_pixbuf());
            about.set_comments("A lightweight, desktop independant software for disks mounting.");
            about.set_copyright("(C) 2010 Boris Albar <elentir@frugalware.org>");
            about.set_website("http://elentir.sleipnir.fr");
            about.run();
            about.hide();
        }

        private Gtk.ImageMenuItem new_disk_menuitem(DeviceDesc desc)
        {
            Gtk.ImageMenuItem imgitem = new Gtk.ImageMenuItem.with_label(desc.label);
            if(desc.is_optical)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("media-cdrom"),
                                            Gtk.IconSize.SMALL_TOOLBAR));
            else
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("drive-removable-media"),
                                            Gtk.IconSize.SMALL_TOOLBAR));

            var submenu = new Gtk.Menu();
            imgitem.set_submenu(submenu);

            Gtk.ImageMenuItem item;
            if(desc.is_audio == true) {
                item = new Gtk.ImageMenuItem.with_label("Play");
                item.set_image(new Image.from_stock(Gtk.Stock.MEDIA_PLAY, Gtk.IconSize.SMALL_TOOLBAR));
                item.set_data<ObjectPath>("path", desc.path);
                item.activate.connect((source) => {
                    ObjectPath path = source.get_data<ObjectPath>("path");
                    play_from_menu(path);
                });
                submenu.append(item);
            } else if(desc.is_blank == true) {
                item = new Gtk.ImageMenuItem.with_label("New Disk");
                item.set_image(new Image.from_stock(Gtk.Stock.CDROM, Gtk.IconSize.SMALL_TOOLBAR));
                item.set_data<ObjectPath>("path", desc.path);
                item.activate.connect((source) => {
                    ObjectPath path = source.get_data<ObjectPath>("path");
                    blank_from_menu(path);
                });
                submenu.append(item);
            } else {
                /* Add mount and umount entries in the submenu */
                item = new Gtk.ImageMenuItem.with_label("Mount");
                item.set_image(new Image.from_stock(Gtk.Stock.APPLY, Gtk.IconSize.SMALL_TOOLBAR));
                item.set_data<ObjectPath>("path", desc.path);
                item.activate.connect((source) => {
                    ObjectPath path = source.get_data<ObjectPath>("path");
                    mount_from_menu(path);
                });
                submenu.append(item);

                item = new Gtk.ImageMenuItem.with_label("Umount");
                item.set_image(new Image.from_stock(Gtk.Stock.CLOSE, Gtk.IconSize.SMALL_TOOLBAR));
                item.set_data<ObjectPath>("path", desc.path);
                item.activate.connect((source) => {
                    ObjectPath path = source.get_data<ObjectPath>("path");
                    umount_from_menu(path);
                });
                submenu.append(item);

                item = new Gtk.ImageMenuItem.with_label("Open");
                item.set_image(new Image.from_stock(Gtk.Stock.OPEN, Gtk.IconSize.SMALL_TOOLBAR));
                item.set_data<ObjectPath>("path", desc.path);
                item.activate.connect((source) => {
                    ObjectPath path = source.get_data<ObjectPath>("path");
                    open_from_menu(path);
                });
                submenu.append(item);
            }

            return imgitem;
        }

        private void popup_menu_callback(uint button, uint activate_time)
        {
            Gtk.Menu mainmenu = new Gtk.Menu();

            foreach (DeviceDesc elem in listdisks) {
                mainmenu.append(new_disk_menuitem(elem));
            }

            SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();
            mainmenu.append(separator);

            ImageMenuItem menuConfig = new ImageMenuItem.from_stock(Gtk.Stock.PREFERENCES, null);
            menuConfig.activate.connect((source) => {
                Configurator configurator = new Configurator();
                configurator.show_all();
            });
            mainmenu.append(menuConfig);

            ImageMenuItem menuAbout = new ImageMenuItem.from_stock(Gtk.Stock.ABOUT, null);
            menuAbout.activate.connect(about_clicked);
            mainmenu.append(menuAbout);

            ImageMenuItem menuExit = new ImageMenuItem.from_stock(Gtk.Stock.QUIT, null);
            menuExit.activate.connect((source) => {
                if(UEvt.powertray != null || UEvt.cputray != null) {
                    this.visible = false;
                    UEvt.diskstray = null;
                } else {
                    UEvt.on_exit(0);
                }});
            mainmenu.append(menuExit);

            mainmenu.show_all();
            mainmenu.popup(null,null, null, button, activate_time);
        }
    }
}