/* uevt-brightness-helper.vala for UEvt
 *
 * Copyright (c) 2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Posix;

namespace UEvtBrightnessHelper
{
    /* available kernel interfaces in priority order */
    static const string[] backlight_interfaces = {
        "nv_backlight",
        "asus_laptop",
        "toshiba",
        "eeepc",
        "thinkpad_screen",
        "acpi_video1",
        "mbp_backlight",
        "acpi_video0",
        "fujitsu-laptop",
        "sony",
        "samsung"
    };

    private string uevt_backlight_helper_get_best_backlight()
    {
        foreach (string blif in backlight_interfaces) {
            if(FileUtils.test("/sys/class/backlight/%s".printf(blif),
                                                FileTest.EXISTS) &&
                FileUtils.test("/sys/class/backlight/%s".printf(blif),
                                                    FileTest.IS_DIR))
                    return blif;
        }

        return "";
    }

    private string uevt_backlight_helper_get_max_brightness(string blif)
    {
        string freq = "";

        File file = File.new_for_path("/sys/class/backlight/%s/max_brightness"
                                     .printf(blif));

        try {
            var dis = new DataInputStream(file.read());
            freq = dis.read_line(null);
        } catch (Error e) {
            message("Could not get the maximum backlight value : %s", e.message);
        }

        return freq;
    }

    private string uevt_backlight_helper_get_brightness(string blif)
    {
        string freq = "";

        File file = File.new_for_path("/sys/class/backlight/%s/actual_brightness"
                                     .printf(blif));

        try {
            var dis = new DataInputStream(file.read());
            freq = dis.read_line(null);
        } catch (Error e) {
            message("Could not get the actual backlight value : %s\n", e.message);
        }

        return freq;
    }

    private void uevt_backlight_helper_set_brightness(string blif, int val)
    {
        int maxvalue = int.parse(uevt_backlight_helper_get_max_brightness(blif));

        if(val < 0 || val > maxvalue) {
            message("Value out of bounds : %d\n", val);
            return;
        }

        FILE file = FILE.open("/sys/class/backlight/%s/brightness"
                              .printf(blif), "w");
        file.puts(val.to_string());

        return;
    }

    private void uevt_backlight_helper_set_min_brightness(string blif)
    {
        FILE file = FILE.open("/sys/class/backlight/%s/brightness"
                              .printf(blif), "w");
        file.puts("0");

        return;
    }

    private void uevt_backlight_helper_set_max_brightness(string blif)
    {
        int maxvalue = int.parse(uevt_backlight_helper_get_max_brightness(blif));

        FILE file = FILE.open("/sys/class/backlight/%s/brightness"
                              .printf(blif), "w");
        file.puts(maxvalue.to_string());

        return;
    }

    private void uevt_backlight_helper_set_up_brightness(string blif)
    {
        int maxvalue = int.parse(uevt_backlight_helper_get_max_brightness(blif));
        int curvalue = int.parse(uevt_backlight_helper_get_brightness(blif));

        if(curvalue < maxvalue)
            curvalue++;

        FILE file = FILE.open("/sys/class/backlight/%s/brightness"
                              .printf(blif), "w");
        file.puts(curvalue.to_string());

        return;
    }

    private void uevt_backlight_helper_set_down_brightness(string blif)
    {
        int curvalue = int.parse(uevt_backlight_helper_get_brightness(blif));

        if(curvalue > 0)
            curvalue--;

        FILE file = FILE.open("/sys/class/backlight/%s/brightness"
                              .printf(blif), "w");
        file.puts(curvalue.to_string());

        return;
    }

    public static int main (string[] args)
    {
        string blif = uevt_backlight_helper_get_best_backlight();

        if(Posix.getuid() != 0) {
            Posix.stdout.printf("This program must be run with root privileges.\n");
            return 1;
        }

        if(blif == "") {
            Posix.stdout.printf("Could not find a proper backlight interface.\n");
            return 1;
        }

        if(args[1] == "show-infos") {
            string max = uevt_backlight_helper_get_max_brightness(blif);
            string cur = uevt_backlight_helper_get_brightness(blif);
            Posix.stdout.printf("Current backlight (using %s interface) : %s/%s\n", blif, cur, max);
        } else if(args[1] == "set") {
                uevt_backlight_helper_set_brightness(blif, int.parse(args[2]));
        } else if(args[1] == "set-min") {
                uevt_backlight_helper_set_min_brightness(blif);
        } else if(args[1] == "set-max") {
                uevt_backlight_helper_set_max_brightness(blif);
        } else if(args[1] == "set-up") {
                uevt_backlight_helper_set_up_brightness(blif);
        } else if(args[1] == "set-down") {
                uevt_backlight_helper_set_down_brightness(blif);
        } else {
            Posix.stdout.printf("Unrecognized option %s\n", args[1]);
            Posix.stdout.printf("Usage : uevt_brightness_helper [OPERATION] [ARGUMENT] \n\n");
            Posix.stdout.printf("    show-infos : Show current backlight and value\n");
            Posix.stdout.printf("    set : set backlight to value x (Argument : backlight value)\n");
            Posix.stdout.printf("    set-min : set backlight to minimum\n");
            Posix.stdout.printf("    set-max : set backlight to maximum\n");
            Posix.stdout.printf("    set-up : increase backlight value of one unit\n");
            Posix.stdout.printf("    set-down : decrease backlight value of one unit\n");
        }

        return 0;
    }
}
