/* power-systray.vala for UEvt
 *
 * Copyright (c) 2010,2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using Gtk;
using ConfigH;
using UEvt;

namespace UPower
{
    public class PowerTray : Gtk.StatusIcon
    {
        private string battery_tooltip;
        private bool screen_status;

        private int max_backlight_value;

        public PowerTray()
        {
            this.set_from_gicon(new ThemedIcon("battery-000"));
            this.set_tooltip_text("UEvt PowerTray");
            this.visible = true;
            this.popup_menu.connect(popup_menu_callback);

            this.screen_status = true;

            this.max_backlight_value = uevt_backlight_get_max_brightness();
        }

        private void update_tooltip_text()
        {
            this.set_tooltip_text("UEvt PowerTray\n" +
                                   battery_tooltip);
        }

        private static string get_battery_icon_index(double percentage)
        {
            if(percentage < 10)
                return "000";
            else if(percentage < 30)
                return "020";
            else if(percentage < 50)
                return "040";
            else if(percentage < 70)
                return "060";
            else if(percentage < 90)
                return "080";

            return "100";
        }

        public void battery_change(uint state, double percent, int64 remains)
        {
            if(state == 1)
                this.set_from_gicon(new ThemedIcon.from_names(
                    {"gpm-battery-%s-charging".printf(get_battery_icon_index(percent)),
                    "battery-%s".printf(get_battery_icon_index(percent))}));
            else
                this.set_from_gicon(new ThemedIcon.from_names(
                    {"gpm-battery-%s".printf(get_battery_icon_index(percent)),
                    "battery-%s-charging".printf(get_battery_icon_index(percent))}));

            int64 minutes = (remains / 60) % 60;
            int64 hours = remains / (60 * 60);

            if(state == 1)
                battery_tooltip = "Battery charging (%.01lf%% - %lld:%02lld remaining)"
                                    .printf(percent, hours, minutes);
            else if(state == 2)
                battery_tooltip = "Battery (%.01lf%% - %lld:%02lld remaining)"
                                    .printf(percent, hours, minutes);
            else if(state == 4)
                battery_tooltip = "Battery fully charged (%lld:%02lld remaining)"
                                    .printf(hours, minutes);

            update_tooltip_text();
        }

        public void battery_remove()
        {
            this.set_from_gicon(new ThemedIcon("battery-missing"));
        }

        private async void suspend_from_menu()
        {
            try {
                yield upowerd.suspend();
            } catch(IOError io) {
                warning("Cannot suspend the system : %s", io.message);
            }
        }

        private async void hibernate_from_menu()
        {
            try {
                yield upowerd.hibernate();
            } catch(IOError io) {
                warning("Cannot hibernate the system : %s", io.message);
            }
        }

        private async void show_power_info(ObjectPath dev_path)
        {
           UPower.Device dev;

           try {
                dev = Bus.get_proxy_sync(BusType.SYSTEM,
                                "org.freedesktop.UPower",
                                            dev_path);
            } catch (IOError io) {
                return;
            }

            PowerInfos infowin = new PowerInfos(dev.vendor, dev.model, dev.serial,
                                dev.Type, dev.technology, dev.capacity, dev.energy,
                                dev.energy_empty, dev.energy_full, dev.energy_full_design,
                                dev.energy_rate, dev.voltage);
            infowin.show_all();
        }

        private Gtk.ImageMenuItem new_power_menuitem(PowerDesc desc)
        {
            Gtk.ImageMenuItem imgitem = new Gtk.ImageMenuItem.with_label(desc.label +
                                    " (%d%c)".printf((int)desc.battery_percentage, '%'));

            if(desc.type == 2 && desc.battery_state == 2)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon.from_names(
                    {"gpm-battery-%s".printf(get_battery_icon_index(desc.battery_percentage)),
                    "battery-%s".printf(get_battery_icon_index(desc.battery_percentage))}),
                    IconSize.SMALL_TOOLBAR));
            else if(desc.type == 2)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon.from_names(
                    {"gpm-battery-%s-charging".printf(get_battery_icon_index(desc.battery_percentage)),
                    "battery-%s-charging".printf(get_battery_icon_index(desc.battery_percentage))}),
                    IconSize.SMALL_TOOLBAR));
            else if(desc.type == 3 && desc.battery_state == 2)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("gpm-ups-%s"
                    .printf(get_battery_icon_index(desc.battery_percentage))), IconSize.SMALL_TOOLBAR));
            else if(desc.type == 3)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("gpm-ups-%s-charging"
                    .printf(get_battery_icon_index(desc.battery_percentage))), IconSize.SMALL_TOOLBAR));
            else if(desc.type == 5)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("gpm-mouse-%s"
                    .printf(get_battery_icon_index(desc.battery_percentage))), IconSize.SMALL_TOOLBAR));
            else if(desc.type == 6)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("gpm-keyboard-%s"
                    .printf(get_battery_icon_index(desc.battery_percentage))), IconSize.SMALL_TOOLBAR));
            else if(desc.type == 7 || desc.type == 8)
                imgitem.set_image(new Image.from_gicon(new ThemedIcon("gpm-phone-%s"
                    .printf(get_battery_icon_index(desc.battery_percentage))), IconSize.SMALL_TOOLBAR));

            imgitem.set_data<ObjectPath>("path", desc.path);

            imgitem.activate.connect((source) => {
                ObjectPath path = source.get_data<ObjectPath>("path");
                show_power_info(path);
            });

            return imgitem;
        }

        private void about_clicked()
        {
            string authors[2];
            authors[0] = "Boris Albar";
            authors[1] = null;

            var about = new AboutDialog();
            about.set_version(PACKAGE_VERSION);
            about.set_program_name("UEvt PowerTray");
            about.set_license(licence);
            about.set_authors(authors);
            Image img = new Image.from_file(ICONDATADIR + "status/battery-charging-080.png");
            about.set_logo(img.get_pixbuf());
            about.set_comments("A lightweight, desktop independant software for power managing.");
            about.set_copyright("(C) 2010-2011 Boris Albar <elentir@frugalware.org>");
            about.set_website("http://elentir.sleipnir.fr");
            about.run();
            about.hide();
        }

        private void set_brightness(int val)
        {
            run_command("pkexec uevt_brightness_helper set %d".printf(val), "", "", true);
        }

        private void popup_menu_callback(uint button, uint activate_time)
        {
            Gtk.Menu mainmenu = new Gtk.Menu();

            foreach (PowerDesc elem in powerhash.get_values()) {
                mainmenu.append(new_power_menuitem(elem));
            }

            SeparatorMenuItem separator = new Gtk.SeparatorMenuItem();
            mainmenu.append(separator);

            if(max_backlight_value > 0) {

                int curvalue = uevt_backlight_get_brightness();
                ImageMenuItem menuBrightness = new Gtk.ImageMenuItem.with_label("Brightness");
                menuBrightness.set_image(new Image.from_gicon(new ThemedIcon("brightness-generic"),
                                                Gtk.IconSize.SMALL_TOOLBAR));
                var submenu = new Gtk.Menu();
                menuBrightness.set_submenu(submenu);
                mainmenu.append(menuBrightness);

                for(int i = 0; i <= max_backlight_value; i++)
                {
                    Gtk.ImageMenuItem item;
                    item = new Gtk.ImageMenuItem.with_label("%d %c".printf(
                            (int)(((float)100/max_backlight_value) * i), 37));
                    item.set_data<int>("value", i);
                    if(i == curvalue)
                        item.set_image(new Image.from_gicon(new ThemedIcon("brightness-generic"),
                                                Gtk.IconSize.SMALL_TOOLBAR));

                    item.activate.connect((source) => {
                        int val = source.get_data<int>("value");
                        set_brightness(val);
                    });
                    submenu.append(item);
                }
            }

            ImageMenuItem menuScreen;
            if(screen_status == true) {
                menuScreen = new ImageMenuItem.with_label("Disable ScreenSaver");
                menuScreen.activate.connect((source) => {
                    run_command_config(config.UEVT_GROUP_POWER,
                        config.UEVT_POWER_DISABLE_SCREENSAVER,
                        "", "", false);
                    screen_status = false;
                });
            } else {
                menuScreen = new ImageMenuItem.with_label("Enable ScreenSaver");
                menuScreen.activate.connect((source) => {
                    run_command_config(config.UEVT_GROUP_POWER,
                        config.UEVT_POWER_ENABLE_SCREENSAVER,
                        "", "", false);
                    screen_status = true;
                });
            }
            menuScreen.set_image(new Image.from_gicon(new ThemedIcon("video-display"), IconSize.SMALL_TOOLBAR));
            mainmenu.append(menuScreen);

            ImageMenuItem menuSuspend = new ImageMenuItem.with_label("Suspend");
            menuSuspend.set_image(new Image.from_gicon(new ThemedIcon("system-suspend"), IconSize.SMALL_TOOLBAR));
            menuSuspend.activate.connect((source) => { suspend_from_menu(); });
            try {
                menuSuspend.set_sensitive(upowerd.can_hibernate && upowerd.suspend_allowed());
            } catch(IOError io) {
                menuSuspend.set_sensitive(false);
            }
            mainmenu.append(menuSuspend);

            ImageMenuItem menuHibernate = new ImageMenuItem.with_label("Hibernate");
            menuHibernate.set_image(new Image.from_gicon(new ThemedIcon("system-suspend-hibernate"), IconSize.SMALL_TOOLBAR));
            menuHibernate.activate.connect((source) => { hibernate_from_menu(); });
            try {
                menuHibernate.set_sensitive(upowerd.can_hibernate && upowerd.hibernate_allowed());
            } catch(IOError io) {
                menuHibernate.set_sensitive(false);
            }
            mainmenu.append(menuHibernate);

            separator = new Gtk.SeparatorMenuItem();
            mainmenu.append(separator);

            ImageMenuItem menuConfig = new ImageMenuItem.from_stock(Gtk.Stock.PREFERENCES, null);
            menuConfig.activate.connect((source) => {
                Configurator configurator = new Configurator();
                configurator.show_all();
            });
            mainmenu.append(menuConfig);

            ImageMenuItem menuAbout = new ImageMenuItem.from_stock(Gtk.Stock.ABOUT, null);
            menuAbout.activate.connect(about_clicked);
            mainmenu.append(menuAbout);

            ImageMenuItem menuExit = new ImageMenuItem.from_stock(Gtk.Stock.QUIT, null);
            menuExit.activate.connect((source) => {
                if(UEvt.diskstray != null || UEvt.cputray != null) {
                    this.visible = false;
                    UEvt.powertray = null;
                } else {
                    UEvt.on_exit(0);
                }});
            mainmenu.append(menuExit);

            mainmenu.show_all();
            mainmenu.popup(null,null, null, button, activate_time);
        }
    }
}
