/* uevt-udisks.vala for UEvt
 *
 * Copyright (c) 2010 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using UEvt;
using ConfigH;

namespace UDisks
{
    struct DeviceDesc {
        public ObjectPath path;
        public string label;
        public bool is_optical;
        public bool is_audio;
        public bool is_blank;
    }

    UDisks.Daemon udisksd;
    List<DeviceDesc?> listdisks;

    void add_disk_item(ObjectPath path, string label, bool is_optical, bool is_audio, bool is_blank)
    {
        DeviceDesc desc = {path, label, is_optical, is_audio, is_blank};
        listdisks.append(desc);
    }

    void remove_disk_item(string path)
    {
        DeviceDesc desc;

        for (int i = 0; i < listdisks.length(); i++) {
            desc = listdisks.nth_data(i);
            if(desc.path == path) {
                run_command_config(config.UEVT_GROUP_DISKS,
                            config.UEVT_ON_DEVICE_REMOVED,
                            desc.label, "", false);

                run_notification("Device Removed",
                    "Device %s has been removed.".printf(desc.label),
                    ICONDATADIR + "actions/dialog-disable.png",
                    config.UEVT_NOTIFY_DISKS);

               listdisks.remove_link(listdisks.nth(i));
            }
        }
    }

    bool has_disk_item(string path)
    {
        DeviceDesc desc;

        for (int i = 0; i < listdisks.length(); i++) {
            desc = listdisks.nth_data(i);
            if(desc.path == path)
                return true;
        }

        return false;
    }

    async void on_device_added_async(ObjectPath dev_path)
    {
        UDisks.Device dev;
        string path;
        bool automounting = true;

       /* connect to the dbus device object */
       try {
            dev = Bus.get_proxy_sync(BusType.SYSTEM,
                            "org.freedesktop.UDisks",
                                        dev_path);
        } catch (IOError io) {
            warning("Could not connect to UDisks/Device: %s", io.message);
            return;
        }

        message("Device Added : %s", dev_path);

        /* Filter devices - first part */
        if (dev.id_usage != "filesystem" || dev.device_is_system_internal ||
        dev.device_presentation_hide || dev.device_presentation_nopolicy)
            return;

        add_disk_item(dev_path, dev.device_file, false, false, false);

        /* Filter devices - second part */
        if(dev.device_is_mounted)
            return;

        /* is it necessary to mount the device automatically ? */
        try {
            automounting = config.get_boolean(config.UEVT_GROUP_DISKS,
                                    config.UEVT_DISKS_AUTOMOUNTING);
        } catch (KeyFileError kf) {
            warning("Error when reading UEvt configuration : %s", kf.message);
        }

        if(automounting) {
            try {
                path = yield dev.filesystem_mount("", {"auth_no_user_interaction"});
            } catch (IOError io1) {
                warning("Cannot mount %s : %s", dev_path, io1.message);
                if(io1.message.contains("org.freedesktop.UDisks.Error.PermissionDenied"))
                    run_notification("Cannot Mount Device",
                        "Permission denied, check your PolicyKit authorizations.",
                        ICONDATADIR + "actions/dialog-disable.png",
                        config.UEVT_NOTIFY_DISKS);
                return;
            }
            run_command_config(config.UEVT_GROUP_DISKS,
                        config.UEVT_ON_DEVICE_MOUNTED,
                        dev.device_file, path, false);

            message("Device %s mounted to %s", dev_path, path);

            run_notification("Removable Device",
                "Device %s has been mounted to %s.".printf(dev.device_file, path),
                ICONDATADIR + "devices/drive-removable-media-2.png",
                config.UEVT_NOTIFY_DISKS);
        } else {
            run_notification("Removable Device",
                "Device %s is connected.".printf(dev.device_file),
                ICONDATADIR + "devices/drive-removable-media-2.png",
                config.UEVT_NOTIFY_DISKS);

            run_command_config(config.UEVT_GROUP_DISKS,
                        config.UEVT_ON_DEVICE_ADDED,
                        dev.device_file, "", false);
        }
    }

    async void on_device_changed_async(ObjectPath dev_path)
    {
        UDisks.Device dev;
        string path;
        bool automounting = true;
        bool is_disc_present;

        /* connect to the dbus device object */
       try {
            dev = Bus.get_proxy_sync(BusType.SYSTEM,
                            "org.freedesktop.UDisks",
                                        dev_path);
        } catch (IOError io) {
            warning("Could not connect to UDisks/Device: %s", io.message);
            return;
        }

        message("Device Changed : %s", dev_path);

        /* Filter devices - first part */
        if (dev.device_presentation_hide || dev.device_presentation_nopolicy)
            return;

        is_disc_present = has_disk_item(dev_path);

        if(!is_disc_present && dev.device_is_optical_disc &&
                                dev.device_is_media_available) {
            add_disk_item(dev_path, dev.device_file, true,
						dev.optical_disc_num_audio_tracks>0,
						dev.optical_disc_is_blank);
        } else if(is_disc_present && !dev.device_is_media_available) {
            remove_disk_item(dev_path);
            return;
        } else {
            return;
        }

        /* Filter devices - second part */
        if(dev.device_is_mounted)
            return;

        /* is it necessary to mount the device automatically ? */
        try {
            automounting = config.get_boolean(config.UEVT_GROUP_DISKS,
                                    config.UEVT_DISKS_AUTOMOUNTING);
        } catch (KeyFileError kf) {
            warning("Error when reading UEvt configuration : %s", kf.message);
        }

        if(dev.optical_disc_num_audio_tracks > 0) { /* audio disk */
            message("Audio disk : %s", dev_path);

            run_command_config(config.UEVT_GROUP_DISKS,
                        config.UEVT_ON_AUDIO_DISC,
                        dev.device_file, "", false);

            run_notification("Optical Disk",
                "An audio disk has been inserted into %s.".printf(dev.device_file),
                ICONDATADIR + "devices/media-cdrom-audio.png",
                config.UEVT_NOTIFY_DISKS);
        } else if(dev.optical_disc_is_blank == true) { /* blank disk */
            message("Blank disk : %s", dev_path);

            run_command_config(config.UEVT_GROUP_DISKS,
                        config.UEVT_ON_BLANK_DISC,
                        dev.device_file, "", false);

            run_notification("Optical Disk",
                "A blank disk has been inserted into %s.".printf(dev.device_file),
                ICONDATADIR + "devices/media-optical-recordable.png",
                config.UEVT_NOTIFY_DISKS);
        } else {
            if(automounting) {
                try {
                    path = yield dev.filesystem_mount("", {"auth_no_user_interaction"});
                } catch (IOError io2) {
                    warning("Cannot mount %s : %s", dev_path, io2.message);
                    if(io2.message.contains("org.freedesktop.UDisks.Error.PermissionDenied"))
                        run_notification("Cannot Mount Device",
                            "Permission denied, check your PolicyKit authorizations.",
                            ICONDATADIR + "actions/dialog-disable.png",
                            config.UEVT_NOTIFY_DISKS);
                    return;
                }

                message("Optical Disk %s mounted to %s.", dev_path, path);

                run_command_config(config.UEVT_GROUP_DISKS,
                            config.UEVT_ON_DEVICE_MOUNTED,
                            dev.device_file, path, false);

                run_notification("Optical Disk",
                    "Disk %s has been mounted to %s.".printf(dev.device_file, path),
                    ICONDATADIR + "devices/media-optical.png",
                    config.UEVT_NOTIFY_DISKS);
            } else {
                run_notification("Optical Disk",
                    "Disk %s is ready.".printf(dev.device_file),
                    ICONDATADIR + "devices/media-optical.png",
                    config.UEVT_NOTIFY_DISKS);

                run_command_config(config.UEVT_GROUP_DISKS,
                            config.UEVT_ON_DEVICE_ADDED,
                            dev.device_file, "", false);
            }
        }
    }

    void on_device_added(ObjectPath device)
    {
        on_device_added_async.begin(device);
    }

    void on_device_removed(ObjectPath device)
    {
        message("Device removed: %s", device);
        remove_disk_item(device);
    }

    void on_device_changed(ObjectPath device)
    {
        on_device_changed_async.begin(device);
    }

    static int load_udisks_plugin()
    {
        try {
            udisksd = Bus.get_proxy_sync(BusType.SYSTEM,
                            "org.freedesktop.UDisks",
                            "/org/freedesktop/UDisks");
        } catch (IOError e) {
            warning("Could not connect to UDisks: %s", e.message);
            return 1;
        }

        /* Test the dbus connection and return if udisks is not running */
        if(udisksd.daemon_version == null) {
            warning("Could not connect to UDisks.");
            return 1;
        }

        message("Connected to UDisks %s\n", udisksd.daemon_version);

        /* Create the list containing all the devices */
        listdisks = new List<DeviceDesc?>();

        try {
            foreach (ObjectPath o in udisksd.enumerate_devices())
                on_device_added_async(o);
        } catch(IOError io) {
            warning("Cannot enumerate initial devices : %s", io.message);
        }

        udisksd.device_added.connect(on_device_added);
        udisksd.device_removed.connect(on_device_removed);
        udisksd.device_changed.connect(on_device_changed);

        return 0;
    }
}
