/* uevt-upower.vala for UEvt
 *
 * Copyright (c) 2010,2011 by Albar Boris <elentir@frugalware.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using GLib;
using UEvt;
using ConfigH;

namespace UPower
{
    UPower.Daemon upowerd;

    struct PowerDesc {
        ObjectPath path;
        string label;
        uint type;
        uint battery_state;
        double battery_percentage;
        bool battery_critical_notified;
        bool battery_low_notified;
    }

    /* we keep the current status up2date, to see if change append */
    bool lid_closed;
    bool line_online;
    double battery_critical;
    double battery_low;
    ObjectPath battery_cur;
    HashTable<string, PowerDesc?> powerhash;

    async void on_power_device_added_async(ObjectPath dev_path)
    {
        UPower.Device dev;
        PowerDesc desc;

        /* connect to the dbus device object */
        try {
            dev = Bus.get_proxy_sync(BusType.SYSTEM,
                            "org.freedesktop.UPower",
                                            dev_path);
        } catch (IOError io) {
            warning("Could not connect to UPower/Device: %s", io.message);
            return;
        }

        /* type of the power device */
        uint type = dev.Type;
        if(type == 1) { /* Line Power providing energy */
            line_online = dev.online;
        } else if(type == 2 && dev.is_present == true) { /* Battery */
            battery_cur = dev_path;
            desc = { dev_path, "Battery", type, dev.state, dev.percentage, false, false };
            powerhash.insert(dev_path, desc);

            /* notify the tray */
            if(powertray != null) {
                if(desc.battery_state == 1)
                    powertray.battery_change(desc.battery_state, desc.battery_percentage, dev.time_to_full);
                else
                    powertray.battery_change(desc.battery_state, desc.battery_percentage, dev.time_to_empty);
            }
        } else if(type == 3) { /* UPS */
            desc = { dev_path, "UPS", type, dev.state, dev.percentage, false, false };
            powerhash.insert(dev_path, desc);
        } else if(type == 5) { /* Mouse */
            desc = { dev_path, "Mouse", type, dev.state, dev.percentage, false, false };
            powerhash.insert(dev_path, desc);
        } else if(type == 6) { /* Keyboard */
            desc = { dev_path, "Keyboard", type, dev.state, dev.percentage, false, false };
            powerhash.insert(dev_path, desc);
        } else if(type == 7) { /* PDA */
            desc = { dev_path, "PDA", type, dev.state, dev.percentage, false, false };
            powerhash.insert(dev_path, desc);
        } else if(type == 8) { /* Phone */
            desc = { dev_path, "Phone", type, dev.state, dev.percentage, false, false };
            powerhash.insert(dev_path, desc);
        }
    }

    async void on_power_device_changed_async(ObjectPath dev_path)
    {
        UPower.Device dev;
        PowerDesc desc;

        /* connect to the dbus device object */
        try {
            dev = Bus.get_proxy_sync(BusType.SYSTEM,
                            "org.freedesktop.UPower",
                                            dev_path);
        } catch (IOError io) {
            warning("Could not connect to UPower/Device: %s", io.message);
            return;
        }

        /* type of the power device */
        uint type = dev.Type;
        if(type == 1) {
            /* Line do not provide energy anymore */
            if(dev.online != line_online) {
                line_online = dev.online;
                if(line_online == true) {
                    run_command_config(config.UEVT_GROUP_POWER,
                        config.UEVT_POWER_ON_LINE_POWER,
                        "", "", false);

                    run_notification("AC Power",
                        "Your computer is running on AC Power.",
                        ICONDATADIR + "devices/ac-adapter.png",
                        config.UEVT_NOTIFY_POWER);
                }
            }
        } else if((type == 2 && dev.is_present == true) || type == 3) { /* Battery */
            desc = powerhash.lookup(dev_path);

            /* Battery State have changed */
            if(desc.battery_state != dev.state) {
                desc.battery_state = dev.state;

                if(desc.battery_state == 1) {
                    /* Battery is now charging */
                    run_notification("Battery",
                        "Your battery is charging.",
                        ICONDATADIR + "status/battery-charging.png",
                        config.UEVT_NOTIFY_POWER);
                } else if(desc.battery_state == 2) {
                    /* Battery is now discharging */
                    run_command_config(config.UEVT_GROUP_POWER,
                                config.UEVT_POWER_ON_BATTERY,
                                "", "", false);

                    run_notification("Battery",
                        "Your computer is running on battery.",
                        ICONDATADIR + "status/battery-good.png",
                        config.UEVT_NOTIFY_POWER);
                } else if(desc.battery_state == 3) {
                    /* Battery is now empty */
                    run_notification("Battery",
                        "Your battery is empty.",
                        ICONDATADIR + "status/battery-empty.png",
                        config.UEVT_NOTIFY_POWER);
                } else if(desc.battery_state == 4 && dev.percentage > 99.0) {
                    /* Battery is now fully charged */
                    run_notification("Battery",
                        "Your battery is fully charged.",
                        ICONDATADIR + "status/battery-full-charging.png",
                        config.UEVT_NOTIFY_POWER);
                }
            }

            /* Battery percentage has been updated */
            if(dev.percentage != desc.battery_percentage) {
                 /* battery percentage update */
                desc.battery_percentage = dev.percentage;
                /* If battery critical */
                if(!desc.battery_critical_notified && desc.battery_percentage < battery_critical) {
                    desc.battery_critical_notified = true;

                    run_command_config(config.UEVT_GROUP_POWER,
                                config.UEVT_POWER_ON_BATTERY_CRITICAL,
                                "", "", false);

                    run_notification("Battery",
                        "Your battery is running critical.\n" +
                        "You have approximately %lld minuts of remaining battery life.".printf(dev.time_to_empty / 60),
                        ICONDATADIR + "status/battery-caution.png",
                        config.UEVT_NOTIFY_POWER);
                } else if(!desc.battery_low_notified && desc.battery_percentage < battery_low) {
                    desc.battery_low_notified = true;

                    run_command_config(config.UEVT_GROUP_POWER,
                                config.UEVT_POWER_ON_BATTERY_LOW,
                                "", "", false);

                    run_notification("Battery",
                        "Your battery is running low.\n" +
                        "You have approximately %lld minuts of remaining battery life.".printf(dev.time_to_empty / 60),
                        ICONDATADIR + "status/battery-low.png",
                        config.UEVT_NOTIFY_POWER);
                }

                /* Re-update battery notifie for further usage */
                if(desc.battery_critical_notified && desc.battery_percentage >= battery_critical)
                    desc.battery_critical_notified = false;
                if(desc.battery_low_notified && desc.battery_percentage >= battery_low)
                    desc.battery_low_notified = false;
            }

            /* notify the tray */
            if(powertray != null) {
                if(desc.battery_state == 1)
                    powertray.battery_change(desc.battery_state, desc.battery_percentage, dev.time_to_full);
                else
                    powertray.battery_change(desc.battery_state, desc.battery_percentage, dev.time_to_empty);
            }

            powerhash.replace(dev_path, desc);
        } else if(type == 5 || type == 6 || type == 7 || type == 8) { /* UPS */
           desc = powerhash.lookup(dev_path);

           /* battery percentage update */
           desc.battery_state = dev.state;
           desc.battery_percentage = dev.percentage;

           /* If battery critical */
           if(!desc.battery_critical_notified && desc.battery_state == 2 &&
                desc.battery_percentage < battery_critical) {
                    desc.battery_critical_notified = true;

                    run_notification("%s".printf(desc.label),
                        "Your %s is running critical.\n".printf(desc.label) +
                        "You have approximately %lld minuts of remaining battery life.".printf(dev.time_to_empty / 60),
                        ICONDATADIR + "status/battery-caution.png",
                        config.UEVT_NOTIFY_POWER);
            } else if(!desc.battery_low_notified && desc.battery_state == 2 &&
                        desc.battery_percentage < battery_low) {
                desc.battery_low_notified = true;

                run_notification("%s".printf(desc.label),
                    "Your %s is running low.\n".printf(desc.label) +
                    "You have approximately %lld minuts of remaining battery life.".printf(dev.time_to_empty / 60),
                    ICONDATADIR + "status/battery-low.png",
                    config.UEVT_NOTIFY_POWER);
            }

            /* Re-update battery notifie for further usage */
            if(desc.battery_critical_notified && desc.battery_percentage >= battery_critical)
                desc.battery_critical_notified = false;
            if(desc.battery_low_notified && desc.battery_percentage >= battery_low)
                desc.battery_low_notified = false;

            powerhash.replace(dev_path, desc);
        }
    }

    void on_power_device_added(ObjectPath device)
    {
        on_power_device_added_async.begin(device);
    }

    void on_power_device_removed(ObjectPath device)
    {
        message("Power device removed: %s", device);
        if(battery_cur != null && device == battery_cur && powertray != null) {
            powertray.battery_remove();
            battery_cur = null;
        }
        powerhash.remove(device);
    }

    void on_power_device_changed(ObjectPath device)
    {
        on_power_device_changed_async.begin(device);
    }

    void on_changed()
    {
        bool suspend_on_lid = false;

        if(upowerd.lid_is_present && (upowerd.lid_is_closed != lid_closed)) {
            lid_closed = upowerd.lid_is_closed;
            if(upowerd.lid_is_closed == true) {
                /* Lid has been closed */
                run_command_config(config.UEVT_GROUP_POWER,
                            config.UEVT_POWER_ON_LID_CLOSED,
                            "", "", false);
                try {
                    suspend_on_lid = config.get_boolean(config.UEVT_GROUP_POWER,
                                config.UEVT_POWER_SUSPEND_ON_LID_CLOSED);
                } catch (KeyFileError kf) {
                    warning("Error when reading UEvt configuration : %s", kf.message);
                }
                if(suspend_on_lid == true)
                    upowerd.suspend();
            } else {
                /* Lid has been opened */
                run_command_config(config.UEVT_GROUP_POWER,
                            config.UEVT_POWER_ON_LID_OPENED,
                            "", "", false);
            }
        }
    }

     void on_sleeping()
    {
        run_command_config(config.UEVT_GROUP_POWER,
                            config.UEVT_POWER_ON_SLEEPING,
                            "", "", false);
    }

     void on_resuming()
    {
        run_command_config(config.UEVT_GROUP_POWER,
                            config.UEVT_POWER_ON_RESUMING,
                            "", "", false);
    }

    static int load_upower_plugin()
    {
        try {
            upowerd = Bus.get_proxy_sync(BusType.SYSTEM,
                            "org.freedesktop.UPower",
                            "/org/freedesktop/UPower");
        } catch (IOError e) {
            warning("Could not connect to Upower: %s", e.message);
            return 1;
        }

        /* Test the dbus connection and return if udisks is not running */
        if(upowerd.daemon_version == null) {
            warning("Could not connect to UDisks.");
            return 1;
        }

        message("Connected to Upower %s\n", upowerd.daemon_version);

        powerhash = new HashTable<string, PowerDesc?>(str_hash, str_equal);

        try {
            foreach (ObjectPath o in upowerd.enumerate_devices()) {
                on_power_device_added_async(o);
            }
        } catch(IOError io) {
            warning("Cannot enumerate initial power devices : %s", io.message);
        }

        /* read battery low and critical level from configuration */
        try {
            battery_low = config.get_double(config.UEVT_GROUP_POWER,
                                config.UEVT_POWER_BATTERY_LOW_LVL);
            battery_critical = config.get_double(config.UEVT_GROUP_POWER,
                                config.UEVT_POWER_BATTERY_CRITICAL_LVL);
        } catch (KeyFileError kf) {
            warning("Error when reading UEvt configuration : %s", kf.message);
        }

        upowerd.device_added.connect(on_power_device_added);
        upowerd.device_removed.connect(on_power_device_removed);
        upowerd.device_changed.connect(on_power_device_changed);
        upowerd.changed.connect(on_changed);
        upowerd.sleeping.connect(on_sleeping);
        upowerd.resuming.connect(on_resuming);

        return 0;
    }
}